"use strict";
// SPDX-FileCopyrightText: © 2022 Schneider Electric
//
// SPDX-License-Identifier: LGPL-2.1-only
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = __importDefault(require("node:path"));
const style_dictionary_1 = __importDefault(require("style-dictionary"));
const logger_js_1 = __importDefault(require("../logger.js"));
const quartz_platform_js_1 = require("../tokens-metadata/quartz-platform.js");
const copyright_js_1 = __importDefault(require("./target-platform/all/fileHeader/copyright.js"));
const filter_1 = require("./target-platform/all/filter");
const transform_1 = require("./target-platform/all/transform");
const configurator_js_1 = __importDefault(require("./target-platform/android/configurator.js"));
const configurator_js_2 = __importDefault(require("./target-platform/css/configurator.js"));
const configurator_js_3 = __importDefault(require("./target-platform/ios/configurator.js"));
const ALL_TARGET_PLATFORMS = ['android', 'css', 'ios'];
class StyleDictionaryGenerator {
    cliOptions;
    tokensMetadata;
    copyrightFileHeader;
    targetPlatforms = ALL_TARGET_PLATFORMS;
    constructor(options, tokensMetadata) {
        this.cliOptions = options;
        this.tokensMetadata = tokensMetadata;
        this.copyrightFileHeader = (0, copyright_js_1.default)(tokensMetadata);
        this.parsePlatformOptions();
    }
    generateTargetPlatforms() {
        for (const quartzPlatform of this.tokensMetadata.platforms) {
            logger_js_1.default.newLine();
            logger_js_1.default.start('Generate', quartzPlatform.nameWithPrefix);
            logger_js_1.default.group();
            this.getRunner(quartzPlatform).buildAllPlatforms();
            logger_js_1.default.groupEnd();
        }
        for (const quartzTheme of this.tokensMetadata.themes) {
            logger_js_1.default.newLine();
            logger_js_1.default.start('Generate', quartzTheme.nameWithPrefix);
            logger_js_1.default.group();
            this.getRunner(quartzTheme).buildAllPlatforms();
            logger_js_1.default.groupEnd();
        }
    }
    listGeneratedFiles() {
        let generatedFiles = [];
        for (const targetPlatform of this.targetPlatforms) {
            const configurator = this.getPlatformConfigurator(targetPlatform);
            for (const quartzPlatform of this.tokensMetadata.platforms) {
                const config = configurator.getConfigForQuartzPlatform(quartzPlatform);
                if (config.files) {
                    generatedFiles = [
                        ...generatedFiles,
                        ...config.files.map((file) => node_path_1.default.join(config.buildPath ?? '', file.destination)),
                    ];
                }
                else {
                    throw new Error(`Missing build path or files in the ${quartzPlatform.name} configuration for the ${targetPlatform} export`);
                }
            }
            for (const quartzTheme of this.tokensMetadata.themes) {
                const config = configurator.getConfigForQuartzTheme(quartzTheme);
                if (config.files) {
                    generatedFiles = [
                        ...generatedFiles,
                        ...config.files.map((file) => node_path_1.default.join(config.buildPath ?? '', file.destination)),
                    ];
                }
                else {
                    throw new Error(`Missing build path or files in the ${quartzTheme.name} configuration for the ${targetPlatform} export`);
                }
            }
        }
        return generatedFiles;
    }
    parsePlatformOptions() {
        const platforms = this.cliOptions.platform.split(';');
        this.targetPlatforms = platforms.includes('all')
            ? ALL_TARGET_PLATFORMS
            : platforms.map((value) => value);
    }
    getRunner(source) {
        const config = {
            tokens: source.styleDictionaryTokens,
            platforms: {},
        };
        let customTransforms = [transform_1.flagUndefinedValue];
        let customTransformGroups = [];
        for (const targetPlatform of this.targetPlatforms) {
            const configurator = this.getPlatformConfigurator(targetPlatform);
            customTransforms = [
                ...customTransforms,
                ...configurator.getCustomTransforms(),
            ];
            customTransformGroups = [
                ...customTransformGroups,
                ...configurator.getCustomTransformGroups(),
            ];
            const platformConfig = source instanceof quartz_platform_js_1.QuartzPlatform
                ? configurator.getConfigForQuartzPlatform(source)
                : configurator.getConfigForQuartzTheme(source);
            config.platforms[targetPlatform.valueOf()] = platformConfig;
        }
        const runner = style_dictionary_1.default.extend(config);
        runner.registerFileHeader(this.copyrightFileHeader);
        runner.registerFilter(filter_1.excludeUndefined);
        for (const customTransform of customTransforms) {
            runner.registerTransform(customTransform);
        }
        for (const customTransformGroup of customTransformGroups) {
            runner.registerTransformGroup(customTransformGroup);
        }
        return runner;
    }
    getPlatformConfigurator(target) {
        switch (target) {
            case 'android': {
                return new configurator_js_1.default(this.cliOptions);
            }
            case 'css': {
                return new configurator_js_2.default(this.cliOptions, this.tokensMetadata);
            }
            case 'ios': {
                return new configurator_js_3.default(this.cliOptions);
            }
            default: {
                throw new Error(`Unsupported target platform`);
            }
        }
    }
}
exports.default = StyleDictionaryGenerator;
